/***********************************************************************************************************************
* File Name     : MMS002_SDK_for_Arduino_Sample_Sketch.ino
* Version       : 1.0.0.0
* Device(s)     : Arduino UNO/DUE/...
* Description   : MMS002 sample program (I2C)
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version	Description
*         : 08.12.2023 0.0.0.1	Preliminary Release
*         : 13.12.2023 0.0.0.2	Corrected the calculation formula for "data_ad_tcomp".
*         : 10.06.2024 1.0.0.0	Release
***********************************************************************************************************************/
#include "config.h"
#include <stdio.h>
#include <Wire.h>
#include "ope_class.h"
#include <math.h>

//==============================================================================
// DEFINES AND MACROS for Arduino Settings
//==============================================================================
#if(ARDUINO_VER == UNO)
	#define WIRE Wire
#elif(ARDUINO_VER == DUE)
	#define WIRE Wire1
#elif(ARDUINO_VER == LEONARDO)
	#define WIRE Wire
#elif(ARDUINO_VER == MEGA2560)
	#define WIRE Wire
#else
	#define WIRE Wire
#endif

//==============================================================================
// VARIABLES
//==============================================================================
// instance generation
OPE   airflow; 

static int32_t velcomp_coef_one;
static int32_t velcomp_coef_two;
static int32_t velcomp_coef_three;
static int32_t norm_coef_xm;
static int32_t norm_coef_yp;
static int32_t norm_coef_ym;

static const float value_accracy = 0.15;          //m/s

static const int32_t value_pow2_23 = 8388608;     //2^23
static const int32_t value_pow2_24 = 16777216;    //2^24
static const float value_pow2_27 = 134217728.0;   //2^27
static const float value_pow2_28 = 268435456.0;   //2^28

static unsigned long time_start = 0;
static const long wait_val = 3000;                //msec

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);     //baudrate 115200
  WIRE.begin();
  delay(250);

  //shutdown
  airflow.transition_shutdown();
  delay(10);

  //reset
  airflow.reset_sensor();
  delay(10);

  airflow_switch_coeff_bank();
  airflow_read_parameter();
  airflow_switch_measure_bank();

  Serial.println("data:");
  Serial.print("Tiem[sec]");
  Serial.print(",");
  Serial.print("Airflow[m/s]");
  Serial.print(",");
  Serial.print("Airdirection[deg]");
  Serial.println("");
  time_start = millis();

  // start measurement
  airflow.start_active();
  
}

void airflow_switch_coeff_bank(){

  uint8_t check_addr = 0x6E;
  uint32_t check_ram_field = 0;
  uint8_t banksw_cmd_addnum = 0x00;

  //idle
  airflow.transition_idle();
  
  check_ram_field = airflow.read_ram_data(check_addr);
  delay(15);

  if(0 != check_ram_field){
    banksw_cmd_addnum = 0x04;
  }else{
    banksw_cmd_addnum = 0x00;
  }
  
  airflow.bank_sw(banksw_cmd_addnum);
}


void airflow_switch_measure_bank(){

  uint8_t banksw_cmd_addnum = 0x06;
  airflow.bank_sw(banksw_cmd_addnum);
}


void airflow_read_parameter(){

  uint8_t addr_counter = 0;
  uint8_t read_target_addr = 0x00;
  uint8_t ram_addr_base = 0x60;
  uint8_t id_addr = 0x66;
  int32_t buf[5];
  uint32_t tmp_data;
  uint32_t sensor_id = 0;

  //ID read
  sensor_id = airflow.read_ram_data(id_addr);
  sensor_id = sensor_id >> 4;
  sensor_id = sensor_id & 0x0000FFFF;
  Serial.print("sensor_id");
  Serial.print(",");
  Serial.print(sensor_id);
  Serial.println(",");

  
  for(addr_counter = 0; addr_counter <= 5; addr_counter++){
    read_target_addr = ram_addr_base + addr_counter;
    tmp_data = airflow.read_ram_data(read_target_addr);
    
    if((uint32_t)value_pow2_27 <= tmp_data){
      buf[addr_counter] =  (int32_t)tmp_data - (int32_t)value_pow2_28;
    }else{
      buf[addr_counter] = (int32_t)tmp_data;
    }
  }

  velcomp_coef_one = buf[0];
  velcomp_coef_two = buf[1];
  velcomp_coef_three = buf[2];
  norm_coef_xm = buf[3];
  norm_coef_yp = buf[4];
  norm_coef_ym = buf[5];

  //debug
  Serial.print("velcomp_coef_one");
  Serial.print(",");
  Serial.print("velcomp_coef_two");
  Serial.print(",");
  Serial.print("velcomp_coef_three");
  Serial.print(",");
  Serial.print("norm_coef_xm");
  Serial.print(",");
  Serial.print("norm_coef_yp");
  Serial.print(",");
  Serial.print("norm_coef_ym");
  Serial.println(",");
  
  Serial.print(velcomp_coef_one);
  Serial.print(",");
  Serial.print(velcomp_coef_two);
  Serial.print(",");
  Serial.print(velcomp_coef_three);
  Serial.print(",");
  Serial.print(norm_coef_xm);
  Serial.print(",");
  Serial.print(norm_coef_yp);
  Serial.print(",");
  Serial.print(norm_coef_ym);
  Serial.println(",");

}


void loop() {
  // put your main code here, to run repeatedly: 
  static unsigned long time_now = 0;
  unsigned long  measured_time = 0;
  double elapsed_time_sec;
  
  float vel = 0.0;
  float dir = 0.0;

  uint32_t data_result_zero = 0;
  uint32_t data_result_one = 0;
  uint32_t data_result_two = 0;

  int32_t cal_result_zero = 0;
  int32_t cal_result_one = 0;
  int32_t cal_result_two = 0;

  float data_x_norm = 0.0;  
  float data_y_norm = 0.0;
  float data_x_tcomp = 0.0;
  float data_y_tcomp = 0.0;

  float coeff_alpha = 0.0;
  float coeff_beta = 0.0;
  float coeff_ganma = 0.0;
  float data_ad_tcomp = 0.0;
  float vel_temp = 0.0;

  const float val_norm_one = 1.0;
  
  // read data
  data_result_zero = airflow.read_result_zero();
  data_result_one = airflow.read_result_one();
  data_result_two = airflow.read_result_two();

  if(value_pow2_23 <= data_result_zero){
    cal_result_zero = (int32_t)data_result_zero - value_pow2_24;
  }else{
    cal_result_zero = (int32_t)data_result_zero;
  }
  
  if(value_pow2_23 <= data_result_one){
    cal_result_one = (int32_t)data_result_one - value_pow2_24;
  }else{
    cal_result_one = (int32_t)data_result_one;
  }
  
  if(value_pow2_23 <= data_result_two){
    cal_result_two = (int32_t)data_result_two - value_pow2_24;
  }else{
    cal_result_two = (int32_t)data_result_two;
  }

  // normalize
  if(0 > data_result_zero){
    data_x_norm = (float)cal_result_zero * (val_norm_one + (float)norm_coef_xm/value_pow2_27);
  }else{
    data_x_norm = (float)cal_result_zero * val_norm_one;
  }

  if(0 > data_result_two){
    data_y_norm = (float)cal_result_two * (val_norm_one + (float)norm_coef_ym/value_pow2_27);
  }else{
    data_y_norm = (float)cal_result_two * (val_norm_one + (float)norm_coef_yp/value_pow2_27);
  }

  //normalize_temp
  data_x_tcomp = data_x_norm * (1.0 + (float)cal_result_one / 1000.0);
  data_y_tcomp = data_y_norm * (1.0 + (float)cal_result_one / 1000.0);

  //calcurate
  data_ad_tcomp = ((data_x_tcomp * data_x_tcomp) + (data_y_tcomp * data_y_tcomp)) * pow(2,-12);
  
  coeff_alpha = (float)velcomp_coef_three / (value_pow2_27 * value_pow2_27 * value_pow2_27);  
  coeff_beta = (float)velcomp_coef_two / (value_pow2_27 * value_pow2_27);
  coeff_ganma = (float)velcomp_coef_one / value_pow2_27;
  
  vel_temp = coeff_alpha * pow(data_ad_tcomp,3) + coeff_beta * pow(data_ad_tcomp,2) + coeff_ganma* data_ad_tcomp;
  vel = sqrt(sqrt(vel_temp*2))/pow(2,5); 

  dir = atan((float)data_x_tcomp/(float)data_y_tcomp)/M_PI*180;

  if((0 > data_x_tcomp)&&(0 > data_y_tcomp)){
    dir = dir;
  }else if(0 < data_y_tcomp){
    dir = dir+180;
  }else if((0 < data_x_tcomp)&&(0 > data_y_tcomp)){
    dir = dir+360;
  }else{
    //NO ACTTION
  }

  if(value_accracy > vel){
    dir = 0;
  }
    
  time_now = millis();
  measured_time = time_now - time_start;
  elapsed_time_sec = (double)measured_time/1000;
  Serial.print(elapsed_time_sec);
  Serial.print(",");
  Serial.print(vel);
  Serial.print(",");
  Serial.println(dir);
  
  delay(wait_val);
  
}
